<?php
/*
Plugin Name: Decimal Product Quantity for WooCommerce PRO
Plugin URI: wpgear.xyz/decimal-product-quantity-woo-pro
Description: Decimal product quantity for WooCommerce. (Piece of Product). Min, Max, Step & Default preset. Separate Variation for Variable Products Supported.
Version: 6.21
Author: WPGear
Author URI: http://wpgear.xyz
License: GPLv2
*/

	include_once(__DIR__ .'/includes/functions.php');
	include_once(__DIR__ .'/includes/admin_setup_woo.php');
	include_once(__DIR__ .'/includes/admin_setup_product.php');
	include_once(__DIR__ .'/includes/admin_order.php');

	WooDecimalProductPro_Check_Updated ();

	$WooDecimalProduct_Min_Quantity_Default    	= get_option ('woodecimalproduct_min_qnt_default', 1);  
    $WooDecimalProduct_Step_Quantity_Default   	= get_option ('woodecimalproduct_step_qnt_default', 1); 
	$WooDecimalProduct_Item_Quantity_Default   	= get_option ('woodecimalproduct_item_qnt_default', 1);
	$WooDecimalProduct_Max_Quantity_Default    	= get_option ('woodecimalproduct_max_qnt_default', '');  
	$WooDecimalProduct_Auto_Correction_Quantity	= get_option ('woodecimalproduct_auto_correction_qnt', 1);
	$WooDecimalProduct_ConsoleLog_Debuging		= get_option ('woodecimalproduct_debug_log', 0);
	$WooDecimalProduct_Uninstall_Del_MetaData 	= get_option ('woodecimalproduct_uninstall_del_metadata', 0);	

    /* Минимальное кол-во выбора Товара, Шаг и Значение по-Умолчанию на странице Товара и Корзины.
    ---*PRO-------------------------------------------------------------- */
	add_filter ('woocommerce_quantity_input_args', 'WooDecimalProductPro_quantity_input_args', 10, 2);
    function WooDecimalProductPro_quantity_input_args ($args, $Product) {
        global $WooDecimalProduct_Min_Quantity_Default;
		global $WooDecimalProduct_Step_Quantity_Default;
		global $WooDecimalProduct_Item_Quantity_Default;
		global $WooDecimalProduct_Max_Quantity_Default;
		
		$Product_ID = $Product->get_id();		
		
		$No_MaxEmpty = '';
		
		$Field_Input_Name 	= $args['input_name'];
		$Field_Input_Value 	= $args['input_value']; 		

		if ($Field_Input_Name == 'quantity') {
			// Страница Товара.
			$WooDecimalProduct_QuantityData = WooDecimalProductPro_Get_QuantityData_by_ProductID ($Product_ID, $No_MaxEmpty);
			
			$Min_Qnt = $WooDecimalProduct_QuantityData['min_qnt'];
			$Max_Qnt = $WooDecimalProduct_QuantityData['max_qnt'];
			$Def_Qnt = $WooDecimalProduct_QuantityData['def_qnt'];
			$Stp_Qnt = $WooDecimalProduct_QuantityData['stp_qnt'];	
			
			$args['min_value'] 	= $Min_Qnt;			
			$args['step'] 		= $Stp_Qnt;
			$args['max_value'] 	= $Max_Qnt;				
			
			if ($Field_Input_Value == 1) {
				$args['input_value'] = $Def_Qnt;
			}			
		} else {
			// Корзина. $Field_Input_Name = as: 'cart[4d435ec58f8c *** this is Cart-Item_KEY *** b4e67fa192888c][qty]'
			$Cart_Item_KEY = substr($Field_Input_Name,5,32);
			
			if ($Cart_Item_KEY) {
				$Cart_Item = WC()->cart->get_cart_item ($Cart_Item_KEY);
				
				if ($Cart_Item) {
					$Variation_ID = $Cart_Item['variation_id'];
					
					if ($Variation_ID > 0) {
						$WooDecimalProduct_QuantityData = WooDecimalProductPro_Get_QuantityData_by_VariationID ($Variation_ID, $No_MaxEmpty);						
						
						$Min_Qnt = $WooDecimalProduct_QuantityData['min_qnt'];
						$Max_Qnt = $WooDecimalProduct_QuantityData['max_qnt'];
						$Def_Qnt = $WooDecimalProduct_QuantityData['def_qnt'];
						$Stp_Qnt = $WooDecimalProduct_QuantityData['stp_qnt'];						
						
						$args['min_value'] 	= $Min_Qnt;			
						$args['step'] 		= $Stp_Qnt;
						$args['max_value'] 	= $Max_Qnt;	
					}
				}
			}
		}		

        return $args;
    }
	
    /* Шаг кол-ва выбора Товара на странице Товара. Индивидуально по каждой Вариации.
    ---*PRO-------------------------------------------------------------- */
	add_action ('woocommerce_after_add_to_cart_form', 'WooDecimalProductPro_quantity_Get_Variation_Arg', 999);
	function WooDecimalProductPro_quantity_Get_Variation_Arg () {
		global $WooDecimalProduct_ConsoleLog_Debuging;
		global $WooDecimalProduct_Auto_Correction_Quantity;
		global $product;
		
		$Parent_ID = $product->get_parent_id();
		
		if ($Parent_ID > 0) {
			// Вариативный Товар.
			$Product_ID = $Parent_ID;
		} else {
			// Простой Товар.
			$Product_ID = $product->get_id();
		}		
		
		if ($Product_ID) {	
			$No_MaxEmpty = '-1';	// Unlimited
			$WooDecimalProduct_QuantityData = WooDecimalProductPro_Get_QuantityData_by_ProductID ($Product_ID, $No_MaxEmpty);
			
			$Min_Qnt 		= $WooDecimalProduct_QuantityData['min_qnt'];
			$Max_Qnt 		= $WooDecimalProduct_QuantityData['max_qnt'];
			$Def_Qnt 		= $WooDecimalProduct_QuantityData['def_qnt'];
			$Stp_Qnt 		= $WooDecimalProduct_QuantityData['stp_qnt'];				
			$QNT_Precision 	= $WooDecimalProduct_QuantityData['precision'];	

			ob_start();
			?>
			<script>
				jQuery(document).ready(function(){
					var WooDecimalProduct_ConsoleLog_Debuging = <?php echo $WooDecimalProduct_ConsoleLog_Debuging; ?>;
					WooDecimalProductQNT_ConsoleLog_Debuging ('WooDecimalProductQuantity-PRO JS Check Quantity - loaded');
					
					var WDCQW_Ajax_URL 					= '<?php echo admin_url ('admin-ajax.php');?>';
					
					var WooDecimalProduct_Min_Qnt 		= <?php echo $Min_Qnt; ?>;
					var WooDecimalProduct_Max_Qnt 		= <?php echo $Max_Qnt; ?>;
					var WooDecimalProduct_Default_Qnt 	= <?php echo $Def_Qnt; ?>;
					var WooDecimalProduct_Step_Qnt 		= <?php echo $Stp_Qnt; ?>;
					var WooDecimalProduct_QNT_Precision	= <?php echo $QNT_Precision; ?>;				
					var WooDecimalProduct_Product_ID 	= <?php echo $Product_ID; ?>;
					WooDecimalProductQNT_ConsoleLog_Debuging ('Product_ID: ' + WooDecimalProduct_Product_ID);
					
					var Variation_ID = 0;

					var WooDecimalProduct_Auto_Correction = '<?php echo $WooDecimalProduct_Auto_Correction_Quantity;?>';				
					WooDecimalProductQNT_ConsoleLog_Debuging ('Auto_Correction: ' + WooDecimalProduct_Auto_Correction);	

					var Element_Input_Quantity = jQuery("input[name=quantity]");

					jQuery("<span id='decimal_product_quantity_for_woo_processing' style='display: none; margin-top: 10px;'>... please wait ...</span>").insertAfter(Element_Input_Quantity);
					var Element_Span_Processing = jQuery("span[id='decimal_product_quantity_for_woo_processing']");

					var Element_Button_Submit = jQuery("button.single_add_to_cart_button");				
					
					var Variation;			
					
					// AutoCorrection
					if (WooDecimalProduct_Auto_Correction) {
						// On Change Quantity.
						jQuery (document).on('change','[name=quantity]',function(e) {
							// Get Variation Data
							WooDecimalProductQNT_ConsoleLog_Debuging ('Change Quantity');					
							WooDecimalProductQNT_ConsoleLog_Debuging (e);
							
							var WooDecimalProduct_QNT_Msg = '';
							
							var WooDecimalProduct_QNT_Input = Element_Input_Quantity.val();
							WooDecimalProduct_QNT_Input = Number(WooDecimalProduct_QNT_Input);
							WooDecimalProductQNT_ConsoleLog_Debuging ('Input: ' + WooDecimalProduct_QNT_Input);
							
							var WooDecimalProduct_QNT_Input_Normal = Number(WooDecimalProduct_QNT_Input.toFixed(WooDecimalProduct_QNT_Precision));
							WooDecimalProductQNT_ConsoleLog_Debuging ('*Input: ' + WooDecimalProduct_QNT_Input_Normal);
							
							WooDecimalProduct_Step_Qnt = Number(WooDecimalProduct_Step_Qnt);

							var WooDecimalProduct_QNT_Input_DivStep = Number((WooDecimalProduct_QNT_Input_Normal / WooDecimalProduct_Step_Qnt).toFixed(WooDecimalProduct_QNT_Precision));
							WooDecimalProductQNT_ConsoleLog_Debuging ('Input_DivStep: ' + WooDecimalProduct_QNT_Input_DivStep);
							
							var WooDecimalProduct_QNT_Input_DivStep_PartInt = WooDecimalProduct_QNT_Input_DivStep.toString();
							WooDecimalProduct_QNT_Input_DivStep_PartInt = WooDecimalProduct_QNT_Input_DivStep_PartInt.split('.');
							WooDecimalProduct_QNT_Input_DivStep_PartInt = WooDecimalProduct_QNT_Input_DivStep_PartInt[0];
							WooDecimalProduct_QNT_Input_DivStep_PartInt = Number(WooDecimalProduct_QNT_Input_DivStep_PartInt);
							WooDecimalProductQNT_ConsoleLog_Debuging ('Input_DivStep_PartInt: ' + WooDecimalProduct_QNT_Input_DivStep_PartInt);				
							
							var WooDecimalProduct_QNT_Input_Check = Number((WooDecimalProduct_QNT_Input_DivStep_PartInt * WooDecimalProduct_Step_Qnt).toFixed(WooDecimalProduct_QNT_Precision));
							WooDecimalProductQNT_ConsoleLog_Debuging ('Check: ' + WooDecimalProduct_QNT_Input_Check);
							
							var WooDecimalProduct_QNT_Valid = WooDecimalProduct_QNT_Input_Normal;
							
							// Check Validation	
							if (WooDecimalProduct_QNT_Input_Normal != WooDecimalProduct_QNT_Input_Check) {
								WooDecimalProductQNT_ConsoleLog_Debuging ('Validation...');
								
								WooDecimalProduct_QNT_Valid = Number((WooDecimalProduct_QNT_Input_Check + WooDecimalProduct_Step_Qnt).toFixed(WooDecimalProduct_QNT_Precision));
								WooDecimalProductQNT_ConsoleLog_Debuging ('Valid: ' + WooDecimalProduct_QNT_Valid);							
								
								WooDecimalProduct_QNT_Msg = WooDecimalProduct_QNT_Input_Normal + ' - No valid value. Auto correction nearest valid value: ' + WooDecimalProduct_QNT_Valid;
							} 
							
							// Check Max.
							if (WooDecimalProduct_Max_Qnt != '-1') {
								WooDecimalProductQNT_ConsoleLog_Debuging ('Check Max...');
								
								if (WooDecimalProduct_QNT_Valid > WooDecimalProduct_Max_Qnt) {
									var WooDecimalProduct_QNT_Input_PartInt = Math.trunc (WooDecimalProduct_Max_Qnt / WooDecimalProduct_Step_Qnt);
									
									WooDecimalProduct_QNT_Valid = Number((WooDecimalProduct_QNT_Input_PartInt * WooDecimalProduct_Step_Qnt).toFixed(WooDecimalProduct_QNT_Precision));

									WooDecimalProduct_QNT_Msg = WooDecimalProduct_QNT_Input_Normal + ' - More than the maximum allowed for this Product. Auto correction to Max: ' + WooDecimalProduct_QNT_Valid;									
								}									
							}						
							
							if (WooDecimalProduct_QNT_Msg != '') {
								Element_Input_Quantity.val(WooDecimalProduct_QNT_Valid);
								
								alert (WooDecimalProduct_QNT_Msg);							
							} else {
								if (WooDecimalProduct_QNT_Input_Normal != WooDecimalProduct_QNT_Input) {
									Element_Input_Quantity.val(WooDecimalProduct_QNT_Input_Normal);
								}
							}
							WooDecimalProductQNT_ConsoleLog_Debuging ('-------------'); 			
						});					
					}
					
					// On Change Variation
					jQuery("input[name='variation_id']").change(function(e) {
						Variation_ID = e.target.value;					
						
						if (Variation_ID == '') {
							// Запрещаем изменять Количество если не выбрана Вариация.
							Element_Input_Quantity.attr('disabled', true);
						} else {
							WooDecimalProductQNT_ConsoleLog_Debuging ('--- Variation_ID: ' + Variation_ID + '---');
							
							Element_Input_Quantity.attr('disabled', false);

							// Get Variation Data.
							if (Variation_ID) {
								Element_Input_Quantity.css('display', 'none');
								Element_Span_Processing.css('display', 'inline-block');
								Element_Button_Submit.css('display', 'none');

								var Product_ID = jQuery("input[name='product_id']").val();
								
								// Get Variation Step.
								var WDCQW_Ajax_Data = {
									'action': 'decimal_product_quantity_for_woo',
									'mode': 'get_variation_args',
									'product_id': Product_ID,
									'variation_id': Variation_ID
								};

								jQuery.ajax({
									type:"POST",
									url: WDCQW_Ajax_URL,
									dataType: 'json',
									data: WDCQW_Ajax_Data,
									cache: false,
									success: function(jsondata) {
										var Obj_Request = jsondata;	
										
										var Status	= Obj_Request.status;
										var Answer 	= Obj_Request.answer;
										Variation	= Obj_Request.variation;
										
										WooDecimalProductQNT_ConsoleLog_Debuging (Obj_Request);					

										if (Answer) {
											WooDecimalProduct_Min_Qnt 		= Variation.min;
											WooDecimalProduct_Step_Qnt		= Variation.step;												
											WooDecimalProduct_Default_Qnt	= Variation.set;
											WooDecimalProduct_Max_Qnt		= Variation.max;
											WooDecimalProduct_QNT_Precision = Variation.precision;
											
											WooDecimalProduct_QNT_Precision = Number(WooDecimalProduct_QNT_Precision);
											// WooDecimalProduct_QNT_Precision = WooDecimalProduct_QNT_Precision.toFixed(1);
											
											if (WooDecimalProduct_Step_Qnt) {
												WooDecimalProduct_Step_Qnt = Number(WooDecimalProduct_Step_Qnt);
												WooDecimalProduct_Step_Qnt = WooDecimalProduct_Step_Qnt.toFixed(WooDecimalProduct_QNT_Precision);
												
												Element_Input_Quantity.attr('step', WooDecimalProduct_Step_Qnt);
											}
											
											if (WooDecimalProduct_Min_Qnt) {
												WooDecimalProduct_Min_Qnt = Number(WooDecimalProduct_Min_Qnt);
												WooDecimalProduct_Min_Qnt = WooDecimalProduct_Min_Qnt.toFixed(WooDecimalProduct_QNT_Precision);
												
												Element_Input_Quantity.attr('min', WooDecimalProduct_Min_Qnt);
											}
											
											if (WooDecimalProduct_Max_Qnt) {
												WooDecimalProduct_Max_Qnt = Number(WooDecimalProduct_Max_Qnt);
												WooDecimalProduct_Max_Qnt = WooDecimalProduct_Max_Qnt.toFixed(WooDecimalProduct_QNT_Precision);
												
												Element_Input_Quantity.attr('max', WooDecimalProduct_Max_Qnt);
											}
											
											if (WooDecimalProduct_Default_Qnt) {
												WooDecimalProduct_Default_Qnt = Number(WooDecimalProduct_Default_Qnt);
												WooDecimalProduct_Default_Qnt = WooDecimalProduct_Default_Qnt.toFixed(WooDecimalProduct_QNT_Precision);
											
												Element_Input_Quantity.val(WooDecimalProduct_Default_Qnt);
											} else {
												Element_Input_Quantity.val(WooDecimalProduct_Min_Qnt);
											}										

											Element_Span_Processing.css('display', 'none');
											Element_Input_Quantity.css('display', 'inline-block');
											Element_Button_Submit.css('display', 'inline-block');
											
											WooDecimalProductQNT_ConsoleLog_Debuging ('Min_Qnt: ' + WooDecimalProduct_Min_Qnt);
											WooDecimalProductQNT_ConsoleLog_Debuging ('Max_Qnt: ' + WooDecimalProduct_Max_Qnt);
											WooDecimalProductQNT_ConsoleLog_Debuging ('Default_Qnt: ' + WooDecimalProduct_Default_Qnt);
											WooDecimalProductQNT_ConsoleLog_Debuging ('Step_Qnt: ' + WooDecimalProduct_Step_Qnt);
											WooDecimalProductQNT_ConsoleLog_Debuging ('Precision: ' + WooDecimalProduct_QNT_Precision);
										}
									}
								});					
							}												
						}	
					});
					
					// Debug in Browser Colsole
					function WooDecimalProductQNT_ConsoleLog_Debuging (ConsoleLog) {
						if (WooDecimalProduct_ConsoleLog_Debuging) {
							console.log (ConsoleLog);
						}
					}
				});	
			</script>
			<?php
			$Buffer = ob_get_contents();		
			ob_end_clean();	
			
			echo $Buffer;			
		}		
	}	
	
    /* Добавление Товара не со Страницы Товара, а из Каталога (без выбора Количества), с учетом возможного минимального Значения Количества и Количества по-Умолчанию.
	 * \woocommerce\includes\wc-template-functions.php
	 * \woocommerce\templates\loop\add-to-cart.php
    ----------------------------------------------------------------- */	
	add_filter ('woocommerce_loop_add_to_cart_args', 'WooDecimalProductPro_loop_add_to_cart_args', 10, 2);
	function WooDecimalProductPro_loop_add_to_cart_args ($args, $product) {
		global $WooDecimalProduct_Min_Quantity_Default;
		global $WooDecimalProduct_Item_Quantity_Default;

		$Product_ID = $product->get_id();	
		
		if ($Product_ID) {
			$WooDecimalProduct_QuantityData = WooDecimalProductPro_Get_QuantityData_by_ProductID ($Product_ID);

			$Min_Qnt = $WooDecimalProduct_QuantityData['min_qnt'];
			$Max_Qnt = $WooDecimalProduct_QuantityData['max_qnt'];
			$Def_Qnt = $WooDecimalProduct_QuantityData['def_qnt'];
			$Stp_Qnt = $WooDecimalProduct_QuantityData['stp_qnt'];

			$args['quantity'] = $Def_Qnt;
		}		

		return $args;
	}	
	
    /* Проверка условий превышения Максимального количества Товара при попытке добавления в Корзину.
	 * \woocommerce\includes\wc-cart-functions.php
	 * Сценарий "B".
    ----------------------------------------------------------------- */
	add_filter ('woocommerce_add_to_cart_validation', 'WooDecimalProductPro_add_to_cart_validation', 10, 5);
	function WooDecimalProductPro_add_to_cart_validation ($Passed, $Product_ID, $Quantity, $Variation_ID, $Variations) {
		if ($Passed) {
			global $WooDecimalProduct_Max_Quantity_Default;

			$Max_Qnt_Variation = '';

			if ($Variation_ID) {
				$No_MaxEmpty = '';
				
				$Max_Qnt_Variation = WooDecimalProductPro_Get_QuantityData_by_VariationID ($Variation_ID, $No_MaxEmpty);
			}
		
			$Cart = WC()->session->cart;
			
			if (empty($Cart) || !is_array($Cart) || 0 === count($Cart)) {
				return $Passed;
			} else {
				foreach ($Cart as $Item) {
					if (is_array($Item)) {
						$Item_Product_ID 	= isset($Item['product_id']) ? $Item['product_id'] : false;
						$Item_Variation_ID 	= isset($Item['variation_id']) ? $Item['variation_id'] : false;
						
						if ($Max_Qnt_Variation != '') {
							if ($Item_Variation_ID == $Variation_ID) {
								$Item_Quantity = isset($Item['quantity']) ? $Item['quantity'] : false;
								
								if ($Item_Quantity) {
									$Total_Quantity = $Item_Quantity + $Quantity;
									
									if ($Total_Quantity > $Max_Qnt_Variation) {
										$Passed = false;
										
										// $Msg = 'Вы превысили допустимое максимальное количество для этой Вариации. Проверьте корзину.';
										$Msg = 'You have exceeded the allowed Maximum Quantity for this Variation. Check Cart.';
										
										wc_add_notice( __($Msg, 'decimal-product-quantity-for-woocommerce'), 'error');

										return $Passed;									
									}			
								}
							}
						} else {
							// Проверка на общих условиях
							if ($Item_Product_ID == $Product_ID) {
								$Item_Quantity = isset($Item['quantity']) ? $Item['quantity'] : false;
								
								if ($Item_Quantity) {
									$Max_Qnt = get_post_meta ($Product_ID, 'woodecimalproduct_max_qnt', true);
									
									if (!$Max_Qnt) {
										$Max_Qnt = $WooDecimalProduct_Max_Quantity_Default;
									}

									if ($Max_Qnt != '') {
										$Total_Quantity = $Item_Quantity + $Quantity;
										
										if ($Total_Quantity > $Max_Qnt) {
											$Passed = false;
											
											// $Msg = 'Вы превысили допустимое максимальное количество для этого продукта. Проверьте корзину.';
											$Msg = 'You have exceeded the allowed Maximum Quantity for this Product. Check Cart.';
											
											wc_add_notice( __($Msg, 'decimal-product-quantity-for-woocommerce'), 'error');

											return $Passed;									
										}									
									}
								}
							}	
						}
						
					}
				}
			}
		}
		
		return $Passed;
	}	
	
    /* Сообщение о добавлении Товара в Корзину с учетом возможного дробного Значения Количества.
	 * \woocommerce\includes\wc-cart-functions.php
    ----------------------------------------------------------------- */	
	add_filter ('wc_add_to_cart_message_html', 'WooDecimalProductPro_wc_add_to_cart_message_html', 10, 2); 	
	function WooDecimalProductPro_wc_add_to_cart_message_html ($message, $products) {
		$count = 0;
		
		foreach ( $products as $product_id => $qty ) {
			$titles[] = ( $qty > 0 ? $qty . ' &times; ' : '' ) . sprintf( _x( '&ldquo;%s&rdquo;', 'Item name in quotes', 'woocommerce' ), strip_tags( get_the_title( $product_id ) ) );
			$count   += $qty;
		}	
		
		$titles = array_filter( $titles );
		$added_text = sprintf( _n( '%s has been added to your cart.', '%s have been added to your cart.', $count, 'woocommerce' ), wc_format_list_of_items( $titles ) );

		// Output success messages.
		if ( 'yes' === get_option( 'woocommerce_cart_redirect_after_add' ) ) {
			$return_to = apply_filters( 'woocommerce_continue_shopping_redirect', wc_get_raw_referer() ? wp_validate_redirect( wc_get_raw_referer(), false ) : wc_get_page_permalink( 'shop' ) );
			$message = sprintf( '<a href="%s" class="button wc-forward">%s</a> %s', esc_url( $return_to ), esc_html__( 'Continue shopping', 'woocommerce' ), esc_html( $added_text ) );
		} else {
			$message = sprintf( '<a href="%s" class="button wc-forward">%s</a> %s', esc_url( wc_get_page_permalink( 'cart' ) ), esc_html__( 'View cart', 'woocommerce' ), esc_html( $added_text ) );
		}

		if ( has_filter( 'wc_add_to_cart_message' ) ) {
			wc_deprecated_function( 'The wc_add_to_cart_message filter', '3.0', 'wc_add_to_cart_message_html' );
			$message = apply_filters( 'wc_add_to_cart_message', $message, $product_id );
		}	

		return $message;
	}	
	
    /* AJAX Processing
    ---*PRO-------------------------------------------------------------- */
	add_action( 'wp_ajax_decimal_product_quantity_for_woo', 'WooDecimalProductPro_Ajax' );
	add_action( 'wp_ajax_nopriv_decimal_product_quantity_for_woo', 'WooDecimalProductPro_Ajax' );	
    function WooDecimalProductPro_Ajax(){
		include_once ('includes/ajax_processing.php');
    }

	/* Корзина. Авто-Коррекция неправильно введенного Значения Количества.
	----------------------------------------------------------------- */	
	add_action ('woocommerce_before_cart', 'WooDecimalProductPro_before_cart', 1);
	function WooDecimalProductPro_before_cart () {
		global $WooDecimalProduct_ConsoleLog_Debuging;
		global $WooDecimalProduct_Auto_Correction_Quantity;
		
		if ($WooDecimalProduct_Auto_Correction_Quantity) {
			$WooDecimalProduct_Cart = array();
			
			$No_MaxEmpty = '-1';	// Unlimited

			foreach( WC()->cart->get_cart() as $cart_item ){
				$product_id 		= $cart_item['data']->get_id();
				$item_product_id 	= $cart_item['data']->get_parent_id();

				if ($item_product_id > 0) {
					// Вариативный Товар.
					$product_id 		= $item_product_id;
					$item_product_id 	= $cart_item['data']->get_id();
				} else {
					// Простой Товар.
					$item_product_id = $product_id;
				}

				$cart_item_key 	= $cart_item['key'];

				$WooDecimalProduct_Cart[$item_product_id] = $cart_item_key;
				
				$WooDecimalProduct_QuantityData[$item_product_id] = WooDecimalProductPro_Get_QuantityData_by_ProductID ($product_id, $No_MaxEmpty);
			}
		
			ob_start();
			?>
			<script type='text/javascript'>				
				jQuery(document).ready(function(){
					var WooDecimalProduct_ConsoleLog_Debuging = <?php echo $WooDecimalProduct_ConsoleLog_Debuging; ?>;					
					WooDecimalProductQNT_ConsoleLog_Debuging ('WooDecimalProduct-PRO JS Check Cart Quantity - loaded');
					
					var WooDecimalProduct_Cart = <?php echo json_encode($WooDecimalProduct_Cart); ?>;
					WooDecimalProductQNT_ConsoleLog_Debuging (WooDecimalProduct_Cart);
					
					var WooDecimalProduct_QuantityData = <?php echo json_encode($WooDecimalProduct_QuantityData); ?>;
					WooDecimalProductQNT_ConsoleLog_Debuging (WooDecimalProduct_QuantityData);					
		
					jQuery (function ($) {
						$('.woocommerce').on('change', 'input.qty', function(e){
							var WooDecimalProduct_ItemProduct_QNT_Msg = '';
							
							WooDecimalProductQNT_ConsoleLog_Debuging (e);
							
							var WooDecimalProduct_ItemInputID = e.currentTarget.attributes.id.value;
							// consoqle.log('input_id: ' + WooDecimalProduct_ItemInputID);
												
							var WooDecimalProduct_Item_Attr_ProductID = e.currentTarget.attributes.product_id;						
													
							// Добавляем Аттрибуты. (Простой и Вариативный Товары)
							if (typeof WooDecimalProduct_Item_Attr_ProductID == 'undefined' || WooDecimalProduct_Item_Attr_ProductID == false) {
								WooDecimalProductQNT_ConsoleLog_Debuging ('item_product_id: N/A. Init.');
								
								Object.keys(WooDecimalProduct_Cart).forEach(function(key) {
									WooDecimalProductQNT_ConsoleLog_Debuging (key, WooDecimalProduct_Cart[key]);
									
									jQuery("input[name='cart[" + WooDecimalProduct_Cart[key] + "][qty]']").attr('product_id', key);
								});	
							} 

							var WooDecimalProduct_ItemProductID = e.currentTarget.attributes.product_id.value;
							WooDecimalProductQNT_ConsoleLog_Debuging ('item_product_id: ' + WooDecimalProduct_ItemProductID);
							
							var WooDecimalProduct_ItemProduct_QuantityData = WooDecimalProduct_QuantityData[WooDecimalProduct_ItemProductID];
							WooDecimalProductQNT_ConsoleLog_Debuging (WooDecimalProduct_ItemProduct_QuantityData);
							
							var WooDecimalProduct_ItemProduct_Min_Qnt 	= Number(WooDecimalProduct_ItemProduct_QuantityData['min_qnt']);
							var WooDecimalProduct_ItemProduct_Max_Qnt 	= Number(WooDecimalProduct_ItemProduct_QuantityData['max_qnt']);
							var WooDecimalProduct_ItemProduct_Def_Qnt 	= Number(WooDecimalProduct_ItemProduct_QuantityData['def_qnt']);
							var WooDecimalProduct_ItemProduct_Stp_Qnt 	= Number(WooDecimalProduct_ItemProduct_QuantityData['stp_qnt']);
							var WooDecimalProduct_ItemProduct_Precision = Number(WooDecimalProduct_ItemProduct_QuantityData['precision']);
							
							var WooDecimalProduct_ItemProduct_Input = e.currentTarget.value;
							WooDecimalProduct_ItemProduct_Input = Number(WooDecimalProduct_ItemProduct_Input);
							WooDecimalProductQNT_ConsoleLog_Debuging ('Input: ' + WooDecimalProduct_ItemProduct_Input);
				
							var WooDecimalProduct_ItemProduct_Input_Normal = Number(WooDecimalProduct_ItemProduct_Input.toFixed(WooDecimalProduct_ItemProduct_Precision));
							WooDecimalProductQNT_ConsoleLog_Debuging ('*Input: ' + WooDecimalProduct_ItemProduct_Input_Normal);

							var WooDecimalProduct_ItemProduct_DivStep = Number((WooDecimalProduct_ItemProduct_Input_Normal / WooDecimalProduct_ItemProduct_Stp_Qnt).toFixed(WooDecimalProduct_ItemProduct_Precision));
							WooDecimalProductQNT_ConsoleLog_Debuging ('Input_DivStep: ' + WooDecimalProduct_ItemProduct_DivStep);
				
							var WooDecimalProduct_ItemProduct_DivStep_PartInt = WooDecimalProduct_ItemProduct_DivStep.toString();
							WooDecimalProduct_ItemProduct_DivStep_PartInt = WooDecimalProduct_ItemProduct_DivStep_PartInt.split('.');
							WooDecimalProduct_ItemProduct_DivStep_PartInt = WooDecimalProduct_ItemProduct_DivStep_PartInt[0];
							WooDecimalProduct_ItemProduct_DivStep_PartInt = Number(WooDecimalProduct_ItemProduct_DivStep_PartInt);
							WooDecimalProductQNT_ConsoleLog_Debuging ('Input_DivStep_PartInt: ' + WooDecimalProduct_ItemProduct_DivStep_PartInt);				
							
							var WooDecimalProduct_ItemProduct_QNT_Input_Check = Number((WooDecimalProduct_ItemProduct_DivStep_PartInt * WooDecimalProduct_ItemProduct_Stp_Qnt).toFixed(WooDecimalProduct_ItemProduct_Precision));
							WooDecimalProductQNT_ConsoleLog_Debuging ('Check: ' + WooDecimalProduct_ItemProduct_QNT_Input_Check);
							
							var WooDecimalProduct_ItemProduct_QNT_Valid = WooDecimalProduct_ItemProduct_Input_Normal;
							
							// Check Validation
							if (WooDecimalProduct_ItemProduct_Input_Normal != WooDecimalProduct_ItemProduct_QNT_Input_Check) {																
								WooDecimalProduct_ItemProduct_QNT_Valid = Number((WooDecimalProduct_ItemProduct_QNT_Input_Check + WooDecimalProduct_ItemProduct_Stp_Qnt).toFixed(WooDecimalProduct_ItemProduct_Precision));
								WooDecimalProductQNT_ConsoleLog_Debuging ('Valid: ' + WooDecimalProduct_ItemProduct_QNT_Valid);
								
								var WooDecimalProduct_ItemProduct_QNT_Msg = WooDecimalProduct_ItemProduct_Input_Normal + ' - No valid value. Auto correction nearest valid value: ' + WooDecimalProduct_ItemProduct_QNT_Valid;
															
								jQuery ("#" + WooDecimalProduct_ItemInputID).val(WooDecimalProduct_ItemProduct_QNT_Valid);
							}

							// Check Max.
							if (WooDecimalProduct_ItemProduct_Max_Qnt != '-1') {
								if (WooDecimalProduct_ItemProduct_QNT_Valid > WooDecimalProduct_ItemProduct_Max_Qnt) {
									var WooDecimalProduct_ItemProduct_QNT_Input_PartInt = Math.trunc (WooDecimalProduct_ItemProduct_Max_Qnt / WooDecimalProduct_ItemProduct_Stp_Qnt);
									
									WooDecimalProduct_ItemProduct_QNT_Valid = Number((WooDecimalProduct_ItemProduct_QNT_Input_PartInt * WooDecimalProduct_ItemProduct_Stp_Qnt).toFixed(WooDecimalProduct_ItemProduct_Precision));

									WooDecimalProduct_ItemProduct_QNT_Msg = WooDecimalProduct_ItemProduct_Input_Normal + ' - More than the maximum allowed for this Product. Auto correction to Max: ' + WooDecimalProduct_ItemProduct_QNT_Valid;									
								}									
							}	

							if (WooDecimalProduct_ItemProduct_QNT_Msg != '') {
								jQuery ("#" + WooDecimalProduct_ItemInputID).val(WooDecimalProduct_ItemProduct_QNT_Valid);
								
								alert (WooDecimalProduct_ItemProduct_QNT_Msg);
							} else {
								if (WooDecimalProduct_ItemProduct_Input_Normal != WooDecimalProduct_ItemProduct_Input) {
									WooDecimalProductQNT_ConsoleLog_Debuging ('Floating Number - Detected.');
									jQuery ("#" + WooDecimalProduct_ItemInputID).val(WooDecimalProduct_ItemProduct_QNT_Input_Check);
								}
							}
							WooDecimalProductQNT_ConsoleLog_Debuging ('-------------');						
						});
					});	
					
					// Debug in Browser Colsole
					function WooDecimalProductQNT_ConsoleLog_Debuging (ConsoleLog) {
						if (WooDecimalProduct_ConsoleLog_Debuging) {
							console.log (ConsoleLog);
						}
					}					
				});
			</script>
			<?php

			$contents = ob_get_contents();
			ob_end_clean();
			echo $contents;			
		}	
	}	
	
	/* DashBoard. WooCommerce. List Products. Добавляем новые Колонки в Списке Товаров.
	----------------------------------------------------------------- */	
	add_filter ('manage_edit-product_columns', 'WooDecimalProductPro_manage_edit_product_columns');
	function WooDecimalProductPro_manage_edit_product_columns ($Columns) {		
		$New_Columns = array();
		
		foreach ($Columns as $column_name => $column_info) {
			$New_Columns [$column_name] = $column_info;
			
			if ($column_name == 'price' ) {
				$New_Columns['quantity'] = 'Quantity';
			}			
		}		
		
		return $New_Columns;
	}
	
	
	/* DashBoard. WooCommerce. List Products. Заполняем новые Колонки в Списке Товаров.
	----------------------------------------------------------------- */
	add_action ('manage_product_posts_custom_column', 'WooDecimalProductPro_manage_product_posts_custom_column', 10, 2);
	function WooDecimalProductPro_manage_product_posts_custom_column ($Column, $Product_ID) {
		if ($Column == 'quantity') {
			$No_MaxEmpty = '---';
			$WooDecimalProduct_QuantityData = WooDecimalProductPro_Get_QuantityData_by_ProductID ($Product_ID, $No_MaxEmpty);
			
			$Min_Qnt 		= $WooDecimalProduct_QuantityData['min_qnt'];
			$Max_Qnt 		= $WooDecimalProduct_QuantityData['max_qnt'];			
			$Stp_Qnt 		= $WooDecimalProduct_QuantityData['stp_qnt'];
			$Set_Qnt 		= $WooDecimalProduct_QuantityData['def_qnt'];			
			$QNT_Precision 	= $WooDecimalProduct_QuantityData['precision'];				

			$WooDecimalProduct_QuantityRanges = WooDecimalProductPro_Get_QuantityRange_by_VariationID ($Product_ID);
			if ($WooDecimalProduct_QuantityRanges) {
				$Min_Qnt = $WooDecimalProduct_QuantityRanges['min_qnt'];
				$Max_Qnt = $WooDecimalProduct_QuantityRanges['max_qnt'];
				$Stp_Qnt = $WooDecimalProduct_QuantityRanges['stp_qnt'];	
				$Set_Qnt = $WooDecimalProduct_QuantityRanges['set_qnt'];	
			}
			
			echo "Min: $Min_Qnt<br>";
			echo "Max: $Max_Qnt <br>";
			echo "Step: $Stp_Qnt <br>";
			echo "Set: $Set_Qnt";
		}
	}	