<?php
/*
* Decimal Product Quantity for WooCommerce PRO
* functions.php
*/

	/* Check NoPRO Plugin Installed
	---*PRO-------------------------------------------------------------- */		
	function WooDecimalProductPro_Check_Plugin_Installed ($Plugin_Slug = null) {
		$Result = false;
		
		if ($Plugin_Slug) {
			if (! function_exists ('get_plugins')) {
				require_once ABSPATH .'wp-admin/includes/plugin.php';
			}
			
			$Plugins = get_plugins();
			
			foreach ($Plugins as $Plugin) {
				$Plugin_TextDomain = $Plugin['TextDomain'];
				if ($Plugin_TextDomain == $Plugin_Slug) {
					$Result = true;
				}
			}			
		}	
		
		return $Result;
	}
	
    /* Проверка успешного Обновления Пред.Версий.
    ----------------------------------------------------------------- */ 	
	function WooDecimalProductPro_Check_Updated () {
		// Проверка успешной конвертации Названий Мета-Полей для Товаров.
			// woodecimalproduct_min_qnt_default  -> woodecimalproduct_min_qnt
			// woodecimalproduct_step_qnt_product -> woodecimalproduct_step_qnt
			// woodecimalproduct_item_qnt_default -> woodecimalproduct_item_qnt
		$WooDecimalProduct_Updated_PoductMeta = get_option ('woodecimalproduct_updated_poductmeta', false); 
		
		if (!$WooDecimalProduct_Updated_PoductMeta) {
			global $wpdb;
			$PostMeta_Table = $wpdb->prefix .'postmeta';
			
			$Query = "SELECT * FROM $PostMeta_Table WHERE meta_key LIKE 'woodecimalproduct_%'";			
			$Result = $wpdb->get_results ($wpdb->prepare($Query, true));
			
			if ($Result) {
				foreach ($Result as $Meta) {
					$post_id 	= $Meta->post_id;
					$meta_key 	= $Meta->meta_key;
					$meta_value = $Meta->meta_value;
					
					if ($meta_value) {
						if ($meta_key == 'woodecimalproduct_min_qnt_default' || $meta_key == 'woodecimalproduct_step_qnt_product' || $meta_key == 'woodecimalproduct_item_qnt_default') {
							if ($meta_key == 'woodecimalproduct_min_qnt_default') {
								$meta_key_new = 'woodecimalproduct_min_qnt';
							}
							
							if ($meta_key == 'woodecimalproduct_step_qnt_product') {
								$meta_key_new = 'woodecimalproduct_step_qnt';
							}

							if ($meta_key == 'woodecimalproduct_item_qnt_default') {
								$meta_key_new = 'woodecimalproduct_item_qnt';
							}

							// Уже может быть сохранено Новое Значение обновленной пред. версии
							$Meta_Key_New_Exist = get_post_meta ($post_id, $meta_key_new, true);
							
							if (!$Meta_Key_New_Exist) {
								update_post_meta($post_id, $meta_key_new, $meta_value);	
							}
							
							delete_post_meta($post_id, $meta_key);							
						}
					} else {
						delete_post_meta($post_id, $meta_key);
					}
				}
			}
			
			update_option('woodecimalproduct_updated_poductmeta', true);
		}
	}

	/* Товар. Минимальное / Максимально кол-во выбора Товара, Шаг, Значение по-Умолчанию, Максимально-Необходимая Точность
	----------------------------------------------------------------- */	
	function WooDecimalProductPro_Get_QuantityData_by_ProductID ($Product_ID, $No_MaxEmpty = '') {
        global $WooDecimalProduct_Min_Quantity_Default;
		global $WooDecimalProduct_Step_Quantity_Default;
		global $WooDecimalProduct_Item_Quantity_Default;
		global $WooDecimalProduct_Max_Quantity_Default;
		
		$WooDecimalProduct_QuantityData['min_qnt'] = 1;
		$WooDecimalProduct_QuantityData['max_qnt'] = '';
		$WooDecimalProduct_QuantityData['def_qnt'] = 1;
		$WooDecimalProduct_QuantityData['stp_qnt'] = 1;
		
		if ($Product_ID) {
			$Min_Qnt = get_post_meta ($Product_ID, 'woodecimalproduct_min_qnt', true);	// Минимальное Количество для данного Товара	
			$Max_Qnt = get_post_meta ($Product_ID, 'woodecimalproduct_max_qnt', true);  // Максимальное Количество для данного Товара	
			$Def_Qnt = get_post_meta ($Product_ID, 'woodecimalproduct_item_qnt', true);	// Default_Qnt для данного Товара
			$Stp_Qnt = get_post_meta ($Product_ID, 'woodecimalproduct_step_qnt', true);	// Шаг изменения для данного Товара

			if (!$Min_Qnt) {
				$Min_Qnt = $WooDecimalProduct_Min_Quantity_Default;
			}
			
			if (!$Max_Qnt) {
				$Max_Qnt = $WooDecimalProduct_Max_Quantity_Default;
			}
			if ($Max_Qnt == '') {
				$Max_Qnt = $No_MaxEmpty; // '-1' for Unlimited
			}			

			if (!$Def_Qnt) {
				$Def_Qnt = $WooDecimalProduct_Item_Quantity_Default;
			}
			
			if (!$Stp_Qnt) {
				$Stp_Qnt = $WooDecimalProduct_Step_Quantity_Default;
			}
			
			if ($Min_Qnt && $Def_Qnt) {
				if ($Def_Qnt < $Min_Qnt) {
					$Def_Qnt = $Min_Qnt;
				}
			}				

			$WooDecimalProduct_QuantityData['min_qnt'] = $Min_Qnt;
			$WooDecimalProduct_QuantityData['max_qnt'] = $Max_Qnt;
			$WooDecimalProduct_QuantityData['def_qnt'] = $Def_Qnt;
			$WooDecimalProduct_QuantityData['stp_qnt'] = $Stp_Qnt;
			
			// Precision
			$Locale_Info = localeconv();
			$Locale_Delimiter = $Locale_Info['decimal_point'];
			
			$Min_QNT_Precision = strlen (substr (strrchr ($Min_Qnt, $Locale_Delimiter), 1));
			$Def_QNT_Precision = strlen (substr (strrchr ($Def_Qnt, $Locale_Delimiter), 1));
			$Stp_QNT_Precision = strlen (substr (strrchr ($Stp_Qnt, $Locale_Delimiter), 1));
			
			$QNT_Precision = max (array ($Min_QNT_Precision, $Def_QNT_Precision, $Stp_QNT_Precision));

			$WooDecimalProduct_QuantityData['precision'] = $QNT_Precision;
	
			return $WooDecimalProduct_QuantityData;
		}
		
		return $WooDecimalProduct_QuantityData;
	}
	
	/* Вариация. Минимальное / Максимально кол-во выбора Вариации, Шаг, Значение по-Умолчанию, Максимально-Необходимая Точность
	----------------------------------------------------------------- */	
	function WooDecimalProductPro_Get_QuantityData_by_VariationID ($Variation_ID, $No_MaxEmpty = '') {
        global $WooDecimalProduct_Min_Quantity_Default;
		global $WooDecimalProduct_Step_Quantity_Default;
		global $WooDecimalProduct_Item_Quantity_Default;
		global $WooDecimalProduct_Max_Quantity_Default;
		
		$WooDecimalProduct_QuantityData['min_qnt'] = 1;
		$WooDecimalProduct_QuantityData['max_qnt'] = '';
		$WooDecimalProduct_QuantityData['def_qnt'] = 1;
		$WooDecimalProduct_QuantityData['stp_qnt'] = 1;		
		
		if ($Variation_ID) {
			$Min_Qnt_Variation = get_post_meta ($Variation_ID, 'woodecimalproduct_min_qnt', true);
			$Max_Qnt_Variation = get_post_meta ($Variation_ID, 'woodecimalproduct_max_qnt', true);
			$Stp_Qnt_Variation = get_post_meta ($Variation_ID, 'woodecimalproduct_step_qnt', true);
			$Set_Qnt_Variation = get_post_meta ($Variation_ID, 'woodecimalproduct_item_qnt', true);			
			
			$Product = wc_get_product ($Variation_ID);

			$Product_ID = $Product->get_parent_id();

			if ($Product_ID) {
				$WooDecimalProduct_QuantityData = WooDecimalProductPro_Get_QuantityData_by_ProductID ($Product_ID, $No_MaxEmpty);	

				$Min_Qnt_Product = $WooDecimalProduct_QuantityData['min_qnt'];
				$Max_Qnt_Product = $WooDecimalProduct_QuantityData['max_qnt'];
				$Stp_Qnt_Product = $WooDecimalProduct_QuantityData['stp_qnt'];
				$Set_Qnt_Product = $WooDecimalProduct_QuantityData['def_qnt'];
				
				if (! $Min_Qnt_Variation) {
					$Min_Qnt_Variation = $Min_Qnt_Product;
				}
				
				if (! $Max_Qnt_Variation) {
					$Max_Qnt_Variation = $Max_Qnt_Product;
				}
				
				if (! $Stp_Qnt_Variation) {
					$Stp_Qnt_Variation = $Stp_Qnt_Product;
				}
				
				if (! $Set_Qnt_Variation) {
					$Set_Qnt_Variation = $Set_Qnt_Product;
				}				
		
			
				$WooDecimalProduct_QuantityData['min_qnt'] = $Min_Qnt_Variation;
				$WooDecimalProduct_QuantityData['max_qnt'] = $Max_Qnt_Variation;
				$WooDecimalProduct_QuantityData['stp_qnt'] = $Stp_Qnt_Variation;
				$WooDecimalProduct_QuantityData['def_qnt'] = $Set_Qnt_Variation;							
			
				return $WooDecimalProduct_QuantityData;
			}
		}
		
		return $WooDecimalProduct_QuantityData;
	}	

	/* Формируем Диапазон уникальных Значений каждого Параметра по всем Вариациям Товара
	----------------------------------------------------------------- */
	function WooDecimalProductPro_Get_QuantityRange_by_VariationID ($Product_ID) {
		$WooDecimalProduct_QuantityRanges = array ();
		
		$Product = wc_get_product ($Product_ID);
		$Variations_ID = $Product->get_children();
		
		$No_MaxEmpty = '';

		if ($Variations_ID) {
			$WooDecimalProduct_QuantityData = WooDecimalProductPro_Get_QuantityData_by_ProductID ($Product_ID, $No_MaxEmpty);	

			$Min_Qnt_Product = $WooDecimalProduct_QuantityData['min_qnt'];	// Минимальное Количество для данного Товара
			$Max_Qnt_Product = $WooDecimalProduct_QuantityData['max_qnt'];	// Максимальное Количество для данного Товара
			$Stp_Qnt_Product = $WooDecimalProduct_QuantityData['stp_qnt'];	// Шаг изменения для данного Товара			
			$Set_Qnt_Product = $WooDecimalProduct_QuantityData['def_qnt'];	// Default_Qnt для данного Товара			
			
			$Min_QNT_Range = array ();
			$Max_QNT_Range = array ();
			$Stp_QNT_Range = array ();
			$Set_QNT_Range = array ();
			
			foreach ($Variations_ID as $Variation_ID) {
				if ($Variation_ID > 0) {					
					$Min_Qnt = get_post_meta ($Variation_ID, 'woodecimalproduct_min_qnt', true);  // Минимальное Количество для Вариации	
					$Max_Qnt = get_post_meta ($Variation_ID, 'woodecimalproduct_max_qnt', true);  // Максимальное Количество для Вариации
					$Stp_Qnt = get_post_meta ($Variation_ID, 'woodecimalproduct_step_qnt', true); // Шаг изменения для Вариации	
					$Set_Qnt = get_post_meta ($Variation_ID, 'woodecimalproduct_item_qnt', true); // Default_Qnt для Вариации
					
					if (! $Min_Qnt) {
						$Min_Qnt = $Min_Qnt_Product;
					}
					if (! $Max_Qnt) {
						$Max_Qnt = $Max_Qnt_Product;
					}
					if (! $Stp_Qnt) {
						$Stp_Qnt = $Stp_Qnt_Product;
					}
					if (! $Set_Qnt) {
						$Set_Qnt = $Set_Qnt_Product;
					}				
					
					$Min_QNT_Range[] = $Min_Qnt;
					$Max_QNT_Range[] = $Max_Qnt;
					$Stp_QNT_Range[] = $Stp_Qnt;
					$Set_QNT_Range[] = $Set_Qnt;
				}
			}
	
			$Min_QNT_Range_Str = min ($Min_QNT_Range) .'-' .max ($Min_QNT_Range);
			$Max_QNT_Range_Str = min ($Max_QNT_Range) .'-' .max ($Max_QNT_Range);
			$Stp_QNT_Range_Str = min ($Stp_QNT_Range) .'-' .max ($Stp_QNT_Range);
			$Set_QNT_Range_Str = min ($Set_QNT_Range) .'-' .max ($Set_QNT_Range);

			$WooDecimalProduct_QuantityRanges['min_qnt'] = $Min_QNT_Range_Str;
			$WooDecimalProduct_QuantityRanges['max_qnt'] = $Max_QNT_Range_Str;
			$WooDecimalProduct_QuantityRanges['stp_qnt'] = $Stp_QNT_Range_Str;
			$WooDecimalProduct_QuantityRanges['set_qnt'] = $Set_QNT_Range_Str;		
		}

		return $WooDecimalProduct_QuantityRanges;		
	}

	/* Нормализуем дробное число с учетом настроек разделителя
	----------------------------------------------------------------- */	
	function WooDecimalProductPro_Normalize_Number ($Number) {
		$Locale_Info = localeconv();
		$Locale_Delimiter = $Locale_Info['decimal_point'];
		
		$Number = str_replace ('.', $Locale_Delimiter, $Number);
		$Number = str_replace (',', $Locale_Delimiter, $Number);
		
		return $Number;
	}	
	
	/* Добавляем новое описание Ошибки.
	----------------------------------------------------------------- */	
	function WooDecimalProductPro_Add_Errors_Msg ($Error_Msg, $Errors_Msg) {
		$Errors_Msg .= "<li>$Error_Msg</li>";
		
		return $Errors_Msg;
	}	